@@ -7,6 +7,10 @@
  *   allocation of temporary storage once only.
  */
 
+
+#define slot_dup(dest, src, sym)  SET_SLOT(dest, sym, duplicate(GET_SLOT(src, sym)))
+
+
 /**
  * Calculate the length of the parameter vector (historically called "coef"
  * even though these are not coefficients).
@@ -1518,7 +1522,7 @@ SEXP lmer_Crosstab(SEXP flist)
 /** 
  * Calculate and return the fitted values.
  * 
- * @param x pointer to an ssclme object
+ * @param x pointer to an lmer object
  * @param mmats list of model matrices
  * @param useRf pointer to a logical scalar indicating if the random
  * effects should be used
@@ -1568,3 +1572,257 @@ SEXP lmer_fitted(SEXP x, SEXP mmats, SEXP useRf)
     UNPROTECT(1);
     return val;
 }
+
+
+
+
+
+
+
+
+
+/*   EXPERIMENTAL!   EXPERIMENTAL!   EXPERIMENTAL!  */
+
+/** 
+ * Copy an lmer object collapsing the fixed effects slots to the response only.
+ * 
+ * @param x pointer to an lmer object
+ * 
+ * @return a duplicate of x with the fixed effects slots collapsed to the response only
+ */
+SEXP lmer_collapse(SEXP x)
+{
+    SEXP 
+        ans = PROTECT(NEW_OBJECT(MAKE_CLASS("lmer"))),
+	Omega = GET_SLOT(x, Matrix_OmegaSym),
+/*         Dim = GET_SLOT(x, Matrix_DimSym);  won't work, no longer exists */
+	Dim = getAttrib(GET_SLOT(x, Matrix_ZtXSym), R_DimSymbol);
+    int 
+        nf = length(Omega), 
+/*         nz = INTEGER(Dim)[1];  ???   */
+        nz = INTEGER(Dim)[0]; /*  ???   */
+
+
+
+    slot_dup(ans, x, Matrix_flistSym);
+    slot_dup(ans, x, Matrix_permSym);
+    slot_dup(ans, x, Matrix_ParentSym);
+    slot_dup(ans, x, Matrix_DSym);
+    slot_dup(ans, x, Matrix_bVarSym);
+    slot_dup(ans, x, Matrix_LSym);
+    slot_dup(ans, x, Matrix_ZZpOSym);
+    slot_dup(ans, x, Matrix_OmegaSym);
+    slot_dup(ans, x, Matrix_REMLSym);
+
+    slot_dup(ans, x, Matrix_ZtZSym);
+
+    slot_dup(ans, x, Matrix_cnamesSym);
+    slot_dup(ans, x, Matrix_devCompSym);
+    slot_dup(ans, x, Matrix_devianceSym);
+    slot_dup(ans, x, Matrix_ncSym);
+    slot_dup(ans, x, Matrix_GpSym);
+    slot_dup(ans, x, Matrix_statusSym);
+
+    slot_dup(ans, x, Matrix_callSym);
+    slot_dup(ans, x, Matrix_termsSym);
+    slot_dup(ans, x, Matrix_assignSym);
+    slot_dup(ans, x, Matrix_fittedSym);
+    slot_dup(ans, x, Matrix_residualsSym);
+    slot_dup(ans, x, Matrix_frameSym);
+
+/*     Not in ssclme version: */
+/*         RXX = "matrix",  */
+/*         RZX = "matrix",  */
+/*         XtX = "matrix",  */
+/*         ZtX = "matrix",  */
+
+
+/*     So, removing from lmer version as well: */
+
+/*     slot_dup(ans, x, Matrix_RXXSym); */
+/*     slot_dup(ans, x, Matrix_RZXSym); */
+/*     slot_dup(ans, x, Matrix_XtXSym); */
+/*     slot_dup(ans, x, Matrix_ZtXSym); */
+
+/*     What about ZtZ ? */
+
+    INTEGER(GET_SLOT(ans, Matrix_ncSym))[nf] = 1;
+    SET_SLOT(ans, Matrix_XtXSym, allocMatrix(REALSXP, 1, 1));
+    REAL(GET_SLOT(ans, Matrix_XtXSym))[0] = NA_REAL;
+    SET_SLOT(ans, Matrix_RXXSym, allocMatrix(REALSXP, 1, 1));
+    REAL(GET_SLOT(ans, Matrix_RXXSym))[0] = NA_REAL;
+    SET_SLOT(ans, Matrix_ZtXSym, allocMatrix(REALSXP, nz, 1));
+    SET_SLOT(ans, Matrix_RZXSym, allocMatrix(REALSXP, nz, 1));
+    LOGICAL(GET_SLOT(ans, Matrix_statusSym))[0] = 0;
+    UNPROTECT(1);
+    return ans;
+}
+
+
+
+/** 
+ * Compute certain components of the Laplace likelihood approximation 
+ * 
+ * @param x pointer to an lmer object
+ * 
+ * @return log likelihood
+ */
+SEXP lmer_laplace_devComp(SEXP x) {
+    SEXP 
+        ranef = PROTECT(lmer_ranef(x)),
+        bVar = GET_SLOT(x, Matrix_bVarSym),
+        Omg = GET_SLOT(x, Matrix_OmegaSym);
+    int 
+        *nc = INTEGER(GET_SLOT(x, Matrix_ncSym)),
+	*Gp = INTEGER(GET_SLOT(x, Matrix_GpSym)),
+        ione = 1, ntot,
+        i, j, k, nci, ncisqr, nlev, nf = length(Omg);
+    double 
+	*Omega, *bVi, *rani, *tmp, *tmp2, 
+        ans = 0, one = 1, 
+/*         tmp3,  */
+        zero = 0;
+
+
+/*     Rprintf("1. ans = %f\n", ans); */
+
+    for (i = 0; i < nf; i++) {
+        nci = nc[i];
+        ncisqr = nci * nci;
+        nlev = (Gp[i + 1] - Gp[i]) / nci;
+
+        rani = REAL(VECTOR_ELT(ranef, i));
+        bVi = REAL(VECTOR_ELT(bVar, i));
+        Omega = REAL(VECTOR_ELT(Omg, i));
+        tmp = Memcpy(Calloc(ncisqr, double), Omega, ncisqr);
+
+        F77_CALL(dpotrf)("U", &nci, tmp, &nci, &j);
+        if (j)
+            error(_("Leading %d minor of Omega[[%d]] not positive definite"),
+                  j, i + 1);
+        for (j = 0; j < nci; j++) { /* 0.5 * nlev * logDet(Omega_i) */
+            ans += nlev * log(tmp[j * (nci + 1)]); /* (2 * 0.5) since factoring */
+        }
+
+
+/*         Rprintf("2. ans = %f\n", ans); */
+
+
+        /* Also need 
+           \sum b' Omega b = (b' tmp)^2, 
+           where b = rows of rani. b is nlev x nci (FIXME: check)
+
+           Is there a LAPACK call that does this? Couldn't find one,
+           so I'll hand-code it for now.
+
+           The calculation boils down to (for k=1:nlev)
+
+           ans += sum(( tmp %*% b[k,] )^2)
+
+           This way, we can re-use Omega = tmp' tmp
+
+
+           Alternative: 
+           The calculation boils down to (for k=1:nlev)
+
+           ans += \sum_j=1^nci b[k,j]^2 * Omega[j,j];
+           ans += 2 \sum_{j<l} b[k,j] * b[k,l] * Omega[j,l];
+        */
+	ntot = nlev * nci;
+	tmp2 = Calloc(ntot, double);
+	F77_CALL(dgemm)("N", "T", &nlev, &nci, &nci, &one, rani, &nlev,
+			tmp, &nci, &zero, tmp2, &nlev);
+        ans -= 0.5 * F77_CALL(ddot)(&ntot, tmp2, &ione, tmp2, &ione);
+        Free(tmp2);
+
+/*         Rprintf("3. ans = %f\n", ans); */
+
+
+/*         for (k = 0; k < nlev; k++) { */
+/*             for (j = 0; j < nci; j++) { */
+/*                 tmp2 = 0; */
+/*                 for (l = j; l < nci; l++) { */
+/*                     tmp2 += tmp[l * nci + j] * rani[l * nlev + k]; */
+/*                 } */
+/*                 ans += tmp2 * tmp2; */
+/*             } */
+/*         } */
+
+        for (k = 0; k < nlev; k++) {
+            Memcpy(tmp, bVi + k * ncisqr, ncisqr);
+            F77_CALL(dpotrf)("U", &nci, tmp, &nci, &j);
+            if (j)
+                error(_("Leading %d minor of bVar[[%d]][,,%d] not positive definite"),
+                      j, i + 1, k + 1);
+/*             tmp3 = 0; */
+            for (j = 0; j < nci; j++) {
+/*                 Rprintf("\t5. tmp[%d] = %f\n",  */
+/*                         j * (nci + 1), */
+/*                         tmp[j * (nci + 1)]); */
+/*                 tmp3 += log(tmp[j * (nci + 1)]); */
+                ans += log(tmp[j * (nci + 1)]);
+            }
+/*             ans += log(fabs(tmp3)); */
+/*             ans += tmp3; */
+/*             Rprintf("4. ans = %f (tmp3 = %f)\n", ans, tmp3); */
+        }
+        Free(tmp);
+    }
+    UNPROTECT(1);
+    return ScalarReal(ans);
+
+
+
+    /*
+  ranefs <- .Call("lmer_ranef", reducedObj, PACKAGE = "Matrix")
+  ## ans <- ans + reducedObj@devComp[2]/2 # log-determinant of Omega
+
+  Omega <- reducedObj@Omega
+  for (i in seq(along = ranefs))
+  {
+      ## contribution for random effects (get it working,
+      ## optimize later) 
+      ## symmetrize RE variance
+      Omega[[i]] <- Omega[[i]] + t(Omega[[i]])
+      diag(Omega[[i]]) <- diag(Omega[[i]]) / 2
+
+      ## want log of `const det(Omega) exp(-1/2 b'
+      ## Omega b )` i.e., const + log det(Omega) - .5
+      ## * (b' Omega b)
+
+      ## FIXME: need to adjust for sigma^2 for appropriate
+      ## models (easy).  These are all the b'Omega b,
+      ## summed as they eventually need to be.  Think of
+      ## this as sum(rowSums((ranefs[[i]] %*% Omega[[i]])
+      ## * ranefs[[i]]))
+
+      ranef.loglik.det <- nrow(ranefs[[i]]) *
+          determinant(Omega[[i]], logarithm = TRUE)$modulus/2
+      ranef.loglik.re <-
+          -sum((ranefs[[i]] %*% Omega[[i]]) * ranefs[[i]])/2
+      ranef.loglik <- ranef.loglik.det + ranef.loglik.re
+
+      ## Jacobian adjustment
+      log.jacobian <-
+          sum(log(abs(apply(reducedObj@bVar[[i]],
+            3,
+
+            ## next line depends on
+            ## whether bVars are variances
+            ## or Cholesly factors
+
+            ## function(x) sum(diag(x)))
+### bug?            function(x) sum(diag( La.chol( x ) )))
+            function(x) prod(diag( La.chol( x ) )))
+      )))
+
+      ## the constant terms from the r.e. and the final
+      ## Laplacian integral cancel out both being:
+      ## ranef.loglik.constant <- 0.5 * length(ranefs[[i]]) * log(2 * base::pi)
+
+      ans <- ans + ranef.loglik + log.jacobian
+  }
+
+    */
+
+}
